package IndustrialRobot "Example that demonstrates robot system models based on the Manutec r3 robot"
  import SI = Modelica.Units.SI;

  package Components "Library of components of the robot"
    extends Modelica.Icons.Package;

    model MechanicalStructure "Model of the mechanical part of the r3 robot (without animation)"
      parameter Boolean animation = true "= true, if animation shall be enabled";
      parameter Modelica.Units.SI.Mass mLoad(min = 0) = 15 "Mass of load";
      parameter Modelica.Units.SI.Position rLoad[3] = {0, 0.25, 0} "Distance from last flange to load mass>";
      parameter Modelica.Units.SI.Acceleration g = 9.81 "Gravity acceleration";
      Modelica.Units.SI.Angle q[6] "Joint angles";
      Modelica.Units.SI.AngularVelocity qd[6] "Joint speeds";
      Modelica.Units.SI.AngularAcceleration qdd[6] "Joint accelerations";
      Modelica.Units.SI.Torque tau[6] "Joint driving torques";
      Modelica.Mechanics.Rotational.Interfaces.Flange_a axis1 annotation(Placement(transformation(extent = {{-220, -180}, {-200, -160}}, rotation = 0, origin = {0, 50}), visible = true, iconTransformation(origin = {0, 20}, extent = {{-220, -180}, {-200, -160}}, rotation = 0)));
      Modelica.Mechanics.Rotational.Interfaces.Flange_a axis2 annotation(Placement(transformation(extent = {{-220, -120}, {-200, -100}}, rotation = 0, origin = {0, 30}), visible = true, iconTransformation(origin = {0, 20}, extent = {{-220, -120}, {-200, -100}}, rotation = 0)));
      Modelica.Mechanics.Rotational.Interfaces.Flange_a axis3 annotation(Placement(transformation(extent = {{-220, -60}, {-200, -40}}, rotation = 0, origin = {0, 10}), visible = true, iconTransformation(origin = {0, 20}, extent = {{-220, -60}, {-200, -40}}, rotation = 0)));
      Modelica.Mechanics.Rotational.Interfaces.Flange_a axis4 annotation(Placement(transformation(extent = {{-220, 0}, {-200, 20}}, rotation = 0, origin = {0, 30}), visible = true, iconTransformation(origin = {0, 20}, extent = {{-220, 0}, {-200, 20}}, rotation = 0)));
      Modelica.Mechanics.Rotational.Interfaces.Flange_a axis5 annotation(Placement(transformation(extent = {{-220, 60}, {-200, 80}}, rotation = 0, origin = {0, 10}), visible = true, iconTransformation(origin = {0, 20}, extent = {{-220, 60}, {-200, 80}}, rotation = 0)));
      Modelica.Mechanics.Rotational.Interfaces.Flange_a axis6 annotation(Placement(transformation(extent = {{-220, 120}, {-200, 140}}, rotation = 0, origin = {0, -10}), visible = true, iconTransformation(origin = {0, 20}, extent = {{-220, 120}, {-200, 140}}, rotation = 0)));
      inner Modelica.Mechanics.MultiBody.World world(g = g * Modelica.Math.Vectors.length({0, -1, 0}), animateWorld = false, animateGravity = false, enableAnimation = animation) annotation(Placement(transformation(extent = {{20, -20}, {-20, 20}}, rotation = 0, origin = {100, -160}), visible = true));
      Modelica.Mechanics.MultiBody.Joints.Revolute r1(n = {0, 1, 0}, useAxisFlange = true, animation = animation) annotation(Placement(transformation(origin = {-60, -120}, extent = {{-10, -10}, {10, 10}}, rotation = 90), visible = true));
      Modelica.Mechanics.MultiBody.Joints.Revolute r2(n = {1, 0, 0}, useAxisFlange = true, animation = animation) annotation(Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = -270, origin = {0, -80}), visible = true));
      Modelica.Mechanics.MultiBody.Joints.Revolute r3(n = {1, 0, 0}, useAxisFlange = true, animation = animation) annotation(Placement(transformation(origin = {60, -40}, extent = {{-10, -10}, {10, 10}}, rotation = 90), visible = true));
      Modelica.Mechanics.MultiBody.Joints.Revolute r4(n = {0, 1, 0}, useAxisFlange = true, animation = animation) annotation(Placement(transformation(origin = {60, 40}, extent = {{-10, -10}, {10, 10}}, rotation = 90), visible = true));
      Modelica.Mechanics.MultiBody.Joints.Revolute r5(n = {1, 0, 0}, useAxisFlange = true, animation = animation) annotation(Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = -270, origin = {0, 80}), visible = true));
      Modelica.Mechanics.MultiBody.Joints.Revolute r6(n = {0, 1, 0}, useAxisFlange = true, animation = animation) annotation(Placement(transformation(origin = {-60, 120}, extent = {{-10, -10}, {10, 10}}, rotation = 90), visible = true));
      Modelica.Mechanics.MultiBody.Parts.BodyShape b0(r = {0, 0.351, 0}, shapeType = "modelica://IndustrialRobot/DocumentationFiles/0.dxf", r_shape = {0, 0, 0}, lengthDirection = {1, 0, 0}, widthDirection = {0, 1, 0}, length = 0.225, width = 0.3, height = 0.3, color = {0, 0, 255}, animation = animation, animateSphere = false, r_CM = {0, 0, 0}, m = 1) annotation(Placement(transformation(origin = {-30, -160}, extent = {{-10, -10}, {10, 10}}, rotation = -180), visible = true));
      Modelica.Mechanics.MultiBody.Parts.BodyShape b1(r = {0, 0.324, 0.3}, I_22 = 1.16, shapeType = "modelica://IndustrialRobot/DocumentationFiles/1.dxf", lengthDirection = {1, 0, 0}, widthDirection = {0, 1, 0}, length = 0.25, width = 0.15, height = 0.2, animation = animation, animateSphere = false, color = {255, 0, 0}, r_CM = {0, 0, 0}, m = 1) annotation(Placement(transformation(origin = {-30, -100}, extent = {{-10, -10}, {10, 10}}, rotation = 0), visible = true));
      Modelica.Mechanics.MultiBody.Parts.BodyShape b2(r = {0, 0.65, 0}, r_CM = {0.172, 0.205, 0}, m = 56.5, I_11 = 2.58, I_22 = 0.64, I_33 = 2.73, I_21 = -0.46, shapeType = "modelica://IndustrialRobot/DocumentationFiles/2.dxf", r_shape = {0, 0, 0}, lengthDirection = {1, 0, 0}, widthDirection = {0, 1, 0}, length = 0.5, width = 0.2, height = 0.15, animation = animation, animateSphere = false, color = {255, 178, 0}) annotation(Placement(transformation(origin = {30, -60}, extent = {{-10, -10}, {10, 10}}, rotation = 0), visible = true));
      Modelica.Mechanics.MultiBody.Parts.BodyShape b3(r = {0, 0.414, -0.155}, r_CM = {0.064, -0.034, 0}, m = 26.4, I_11 = 0.279, I_22 = 0.245, I_33 = 0.413, I_21 = -0.07, shapeType = "modelica://IndustrialRobot/DocumentationFiles/3.dxf", r_shape = {0, 0, 0}, lengthDirection = {1, 0, 0}, widthDirection = {0, 1, 0}, length = 0.15, width = 0.15, height = 0.15, animation = animation, animateSphere = false, color = {255, 0, 0}) annotation(Placement(transformation(origin = {60, 0}, extent = {{-10, 10}, {10, -10}}, rotation = 90), visible = true));
      Modelica.Mechanics.MultiBody.Parts.BodyShape b4(r = {0, 0.186, 0}, r_CM = {0, 0, 0}, m = 28.7, I_11 = 1.67, I_22 = 0.081, I_33 = 1.67, shapeType = "modelica://IndustrialRobot/DocumentationFiles/4.dxf", r_shape = {0, 0, 0}, lengthDirection = {1, 0, 0}, widthDirection = {0, 1, 0}, length = 0.73, width = 0.1, height = 0.1, animation = animation, animateSphere = false, color = {255, 178, 0}) annotation(Placement(transformation(origin = {30, 60}, extent = {{-10, -10}, {10, 10}}, rotation = -180), visible = true));
      Modelica.Mechanics.MultiBody.Parts.BodyShape b5(r = {0, 0.125, 0}, r_CM = {0, 0, 0}, m = 5.2, I_11 = 1.25, I_22 = 0.81, I_33 = 1.53, r_shape = {0, 0, 0}, lengthDirection = {1, 0, 0}, widthDirection = {0, 1, 0}, length = 0.225, width = 0.075, height = 0.1, animation = animation, animateSphere = false, color = {0, 0, 255}, shapeType = "modelica://IndustrialRobot/DocumentationFiles/5.dxf") annotation(Placement(transformation(origin = {-30, 100}, extent = {{-10, -10}, {10, 10}}, rotation = -180), visible = true));
      Modelica.Mechanics.MultiBody.Parts.BodyShape b6(r = {0, 0, 0}, r_CM = {0.05, 0.05, 0.05}, m = 0.5, shapeType = "modelica://IndustrialRobot/DocumentationFiles/6.dxf", r_shape = {0, 0, 0}, lengthDirection = {1, 0, 0}, widthDirection = {0, 1, 0}, animation = animation, animateSphere = false, color = {0, 0, 255}) annotation(Placement(transformation(origin = {-60, 150}, extent = {{-10, -10}, {10, 10}}, rotation = 90), visible = true));
      Modelica.Mechanics.MultiBody.Parts.BodyShape load(r_CM = rLoad, m = mLoad, r_shape = {0, 0, 0}, widthDirection = {1, 0, 0}, width = 0.05, height = 0.05, color = {255, 0, 0}, lengthDirection = rLoad, length = Modelica.Math.Vectors.length(rLoad), animation = animation) annotation(Placement(transformation(origin = {-60, 180}, extent = {{-10, -10}, {10, 10}}, rotation = 90), visible = true));
      Modelica.Mechanics.MultiBody.Parts.FixedTranslation toA(r = {0.15, 0.0, 1.0}, height = 0.1, width = 0.1) annotation(Placement(visible = true, transformation(origin = {90, -80}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
      Modelica.Mechanics.MultiBody.Parts.FixedTranslation toB(r = {1.34, 0, -0.95}, height = 0.1, width = 0.1) annotation(Placement(visible = true, transformation(origin = {90, -120}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
      Modelica.Mechanics.MultiBody.Parts.FixedTranslation toA1(height = 0.1, r = {0, 0.3, 0}, width = 0.1) annotation(Placement(visible = true, transformation(origin = {120, -30}, extent = {{-10, -10}, {10, 10}}, rotation = -270)));
      Modelica.Mechanics.MultiBody.Parts.BodyBox bodyBox2(color = {100, 200, 300}, height = 0.5, length = 0.3, r = {0, 0, 0}, r_shape = {-0.15, 0.0, 0.0}, width = 0.05) annotation(Placement(visible = true, transformation(origin = {120, -0}, extent = {{-10, -10}, {10, 10}}, rotation = -270)));
      Modelica.Mechanics.MultiBody.Parts.FixedTranslation toB1(height = 0.1, r = {0.0, 1.5, 0.0}, width = 0.1) annotation(Placement(visible = true, transformation(origin = {180, 30}, extent = {{-10, -10}, {10, 10}}, rotation = -270)));
      Modelica.Mechanics.MultiBody.Parts.BodyBox bodyBox3(color = {155, 200, 0}, height = 0.5, length = 0.3, r_shape = {-0.15, 0, 0}, width = 0.05) annotation(Placement(visible = true, transformation(origin = {180, 60}, extent = {{-10, -10}, {10, 10}}, rotation = -270)));
      Modelica.Mechanics.MultiBody.Parts.Fixed Origin annotation(Placement(visible = true, transformation(origin = {-54.249, 0}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
      Modelica.Mechanics.MultiBody.Visualizers.FixedShape GroundVisualizer(shapeType = "modelica://IndustrialRobot/DocumentationFiles/floor.obj", height = 1, length = 1, width = 1, widthDirection = {0, 0, 1}, r_shape = {-3, 0, 0}) annotation(Placement(visible = true, transformation(origin = {-10, 0}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
    equation
      connect(Origin.frame_b, GroundVisualizer.frame_a) annotation(Line(visible = true, origin = {-32.125, 0}, points = {{-12.124, 0}, {12.125, 0}}, color = {95, 95, 95}));
      connect(toB1.frame_a, toB.frame_b) annotation(Line(visible = true, origin = {153.333, -73.333}, points = {{26.667, 93.333}, {26.667, -46.667}, {-53.333, -46.667}}, color = {95, 95, 95}));
      connect(toB.frame_a, world.frame_b) annotation(Line(visible = true, origin = {41.625, -140.784}, points = {{38.375, 20.784}, {-1.625, 20.784}, {-1.625, -19.216}, {38.375, -19.216}}, color = {95, 95, 95}));
      connect(bodyBox3.frame_a, toB1.frame_b) annotation(Line(visible = true, origin = {180, 45}, points = {{0, 5}, {0, -5}}, color = {95, 95, 95}));
      connect(bodyBox2.frame_a, toA1.frame_b) annotation(Line(visible = true, origin = {120, -15}, points = {{0, 5}, {0, -5}}, color = {95, 95, 95}));
      connect(toA1.frame_a, toA.frame_b) annotation(Line(visible = true, origin = {113.333, -66.667}, points = {{6.667, 26.667}, {6.667, -13.333}, {-13.333, -13.333}}, color = {95, 95, 95}));
      connect(toA.frame_a, world.frame_b) annotation(Line(visible = true, origin = {41.625, -120.784}, points = {{38.375, 40.784}, {-1.625, 40.784}, {-1.625, -39.216}, {38.375, -39.216}}, color = {95, 95, 95}));
      connect(b4.frame_b, r5.frame_a) annotation(Line(visible = true, origin = {6.667, 63.333}, points = {{13.333, -3.333}, {-6.667, -3.333}, {-6.667, 6.667}}, color = {95, 95, 95}, thickness = 0.5));
      connect(r4.frame_b, b4.frame_a) annotation(Line(visible = true, origin = {53.333, 56.667}, points = {{6.667, -6.667}, {6.667, 3.333}, {-13.333, 3.333}}, color = {95, 95, 95}, thickness = 0.5));
      connect(r6.frame_b, b6.frame_a) annotation(Line(points = {{0, -5}, {0, 5}}, color = {95, 95, 95}, thickness = 0.5, visible = true, origin = {-60, 135}));
      q = {r1.phi, r2.phi, r3.phi, r4.phi, r5.phi, r6.phi};
      qd = der(q);
      qdd = der(qd);
      tau = {r1.axis.tau, r2.axis.tau, r3.axis.tau, r4.axis.tau, r5.axis.tau, r6.axis.tau};
      connect(load.frame_a, b6.frame_b) annotation(Line(points = {{0, 5}, {0, -5}}, color = {95, 95, 95}, thickness = 0.5, visible = true, origin = {-60, 165}));
      connect(world.frame_b, b0.frame_a) annotation(Line(points = {{73.494, 0.784}, {-23.494, 0.784}, {-23.494, 0.784}, {-26.506, 0.784}}, color = {95, 95, 95}, thickness = 0.5, visible = true, origin = {6.506, -160.784}));
      connect(b0.frame_b, r1.frame_a) annotation(Line(points = {{13.333, -10}, {-6.667, -10}, {-6.667, 20}}, color = {95, 95, 95}, thickness = 0.5, visible = true, origin = {-53.333, -150}));
      connect(b1.frame_b, r2.frame_a) annotation(Line(points = {{-13.333, -3.333}, {6.667, -3.333}, {6.667, 6.667}}, color = {95, 95, 95}, thickness = 0.5, visible = true, origin = {-6.667, -96.667}));
      connect(r1.frame_b, b1.frame_a) annotation(Line(points = {{-6.667, -6.667}, {-6.667, 3.333}, {13.333, 3.333}}, color = {95, 95, 95}, thickness = 0.5, visible = true, origin = {-53.333, -103.333}));
      connect(r2.frame_b, b2.frame_a) annotation(Line(points = {{-6.667, -6.667}, {-6.667, 3.333}, {13.333, 3.333}}, color = {95, 95, 95}, thickness = 0.5, visible = true, origin = {6.667, -63.333}));
      connect(b2.frame_b, r3.frame_a) annotation(Line(points = {{-13.333, -3.333}, {6.667, -3.333}, {6.667, 6.667}}, color = {95, 95, 95}, thickness = 0.5, visible = true, origin = {53.333, -56.667}));
      connect(r2.axis, axis2) annotation(Line(points = {{100, 0}, {-100, 0}}, visible = true, origin = {-110, -80}, color = {64, 64, 64}));
      connect(r1.axis, axis1) annotation(Line(points = {{70, 0}, {-70, 0}}, visible = true, origin = {-140, -120}, color = {64, 64, 64}));
      connect(r3.frame_b, b3.frame_a) annotation(Line(points = {{0, -10}, {0, 10}}, color = {95, 95, 95}, thickness = 0.5, visible = true, origin = {60, -20}));
      connect(b3.frame_b, r4.frame_a) annotation(Line(points = {{0, -10}, {0, 10}}, color = {95, 95, 95}, thickness = 0.5, visible = true, origin = {60, 20}));
      connect(r3.axis, axis3) annotation(Line(points = {{130, 0}, {-130, 0}}, visible = true, origin = {-80, -40}, color = {64, 64, 64}));
      connect(r4.axis, axis4) annotation(Line(points = {{130, 0}, {-130, 0}}, visible = true, origin = {-80, 40}, color = {64, 64, 64}));
      connect(r5.axis, axis5) annotation(Line(points = {{100, 0}, {-100, 0}}, visible = true, origin = {-110, 80}, color = {64, 64, 64}));
      connect(r5.frame_b, b5.frame_a) annotation(Line(points = {{6.667, -6.667}, {6.667, 3.333}, {-13.333, 3.333}}, color = {95, 95, 95}, thickness = 0.5, visible = true, origin = {-6.667, 96.667}));
      connect(b5.frame_b, r6.frame_a) annotation(Line(points = {{13.333, -3.333}, {-6.667, -3.333}, {-6.667, 6.667}}, color = {95, 95, 95}, thickness = 0.5, visible = true, origin = {-53.333, 103.333}));
      connect(r6.axis, axis6) annotation(Line(points = {{70, 0}, {-70, 0}}, visible = true, origin = {-140, 120}, color = {64, 64, 64}));
      annotation(Documentation(info = "<html><div>
<div><span>This model contains the mechanical parts of the industrial robot. It is a modified version of </span><a href=\"modelica://Modelica.Mechanics.MultiBody.Examples.Systems.RobotR3.Utilities.MechanicalStructure\">MechanicalStructure</a><span> from the Modelica Standard Library, with added visualization components.</span></div>
</div></html>"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-200, -200}, {200, 200}}, grid = {2, 2}, initialScale = 0.1), graphics = {Rectangle(visible = true, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid, extent = {{-200, -200}, {200, 200}}), Text(visible = true, extent = {{-200, 200}, {200, 280}}, textString = "%name", fontName = "Arial"), Text(visible = true, origin = {0, 20}, extent = {{-200, -190}, {-140, -150}}, textString = "1", fontName = "Arial"), Text(visible = true, origin = {0, 20}, extent = {{-200, -70}, {-140, -30}}, textString = "3", fontName = "Arial"), Text(visible = true, origin = {0, 20}, extent = {{-200, -130}, {-140, -90}}, textString = "2", fontName = "Arial"), Text(visible = true, origin = {0, 20}, extent = {{-200, 50}, {-140, 90}}, textString = "5", fontName = "Arial"), Text(visible = true, origin = {0, 22}, extent = {{-200, -12}, {-140, 28}}, textString = "4", fontName = "Arial"), Text(visible = true, origin = {0, 20}, extent = {{-198, 110}, {-138, 150}}, textString = "6", fontName = "Arial")}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-200.0, -200.0}, {200.0, 200.0}}, grid = {10, 10}, initialScale = 0.1)));
    end MechanicalStructure;
    annotation(Documentation(info = "<html></html>"));
  end Components;

  model FullRobot "6 degree of freedom robot with path planning, controllers, motors, brakes, gears and mechanics."
    import SI = Modelica.Units.SI;
    parameter SI.Mass mLoad(min = 0) = 15 "Mass of load";
    parameter SI.Position rLoad[3] = {0.1, 0.25, 0.1} "Distance from last flange to load mass";
    parameter SI.Acceleration g = 9.81 "Gravity acceleration";
    parameter SI.Time refStartTime = 0 "Start time of reference motion";
    parameter SI.Time refSwingTime = 0.5 "Additional time after reference motion is in rest before simulation is stopped";
    parameter Real startAngle1(unit = "deg") = 0 "Start angle of axis 1" annotation(Dialog(tab = "Reference", group = "startAngles"));
    parameter Real startAngle2(unit = "deg") = 60.0 "Start angle of axis 2" annotation(Dialog(tab = "Reference", group = "startAngles"));
    parameter Real startAngle3(unit = "deg") = 90 "Start angle of axis 3" annotation(Dialog(tab = "Reference", group = "startAngles"));
    parameter Real startAngle4(unit = "deg") = 0 "Start angle of axis 4" annotation(Dialog(tab = "Reference", group = "startAngles"));
    parameter Real startAngle5(unit = "deg") = 90 "Start angle of axis 5" annotation(Dialog(tab = "Reference", group = "startAngles"));
    parameter Real startAngle6(unit = "deg") = 45 "Start angle of axis 6" annotation(Dialog(tab = "Reference", group = "startAngles"));
    parameter Real endAngle1(unit = "deg") = 120 "End angle of axis 1" annotation(Dialog(tab = "Reference", group = "endAngles"));
    parameter Real endAngle2(unit = "deg") = 20 "End angle of axis 2" annotation(Dialog(tab = "Reference", group = "endAngles"));
    parameter Real endAngle3(unit = "deg") = 60.0 "End angle of axis 3" annotation(Dialog(tab = "Reference", group = "endAngles"));
    parameter Real endAngle4(unit = "deg") = 0 "End angle of axis 4" annotation(Dialog(tab = "Reference", group = "endAngles"));
    parameter Real endAngle5(unit = "deg") = 0 "End angle of axis 5" annotation(Dialog(tab = "Reference", group = "endAngles"));
    parameter Real endAngle6(unit = "deg") = 90 "End angle of axis 6" annotation(Dialog(tab = "Reference", group = "endAngles"));
    parameter SI.AngularVelocity refSpeedMax[6] = {2, 1.5, 5, 3.1, 3.1, 4.1} "Maximum reference speeds of all joints" annotation(Dialog(tab = "Reference", group = "Limits"));
    parameter SI.AngularAcceleration refAccMax[6] = {15, 15, 15, 60, 60, 60} "Maximum reference accelerations of all joints" annotation(Dialog(tab = "Reference", group = "Limits"));
    parameter Real kp1 = 5 "Gain of position controller" annotation(Dialog(tab = "Controller", group = "Axis 1"));
    parameter Real ks1 = 0.5 "Gain of speed controller" annotation(Dialog(tab = "Controller", group = "Axis 1"));
    parameter SI.Time Ts1 = 0.05 "Time constant of integrator of speed controller" annotation(Dialog(tab = "Controller", group = "Axis 1"));
    parameter Real kp2 = 5 "Gain of position controller" annotation(Dialog(tab = "Controller", group = "Axis 2"));
    parameter Real ks2 = 0.5 "Gain of speed controller" annotation(Dialog(tab = "Controller", group = "Axis 2"));
    parameter SI.Time Ts2 = 0.05 "Time constant of integrator of speed controller" annotation(Dialog(tab = "Controller", group = "Axis 2"));
    parameter Real kp3 = 5 "Gain of position controller" annotation(Dialog(tab = "Controller", group = "Axis 3"));
    parameter Real ks3 = 0.5 "Gain of speed controller" annotation(Dialog(tab = "Controller", group = "Axis 3"));
    parameter SI.Time Ts3 = 0.05 "Time constant of integrator of speed controller" annotation(Dialog(tab = "Controller", group = "Axis 3"));
    parameter Real kp4 = 5 "Gain of position controller" annotation(Dialog(tab = "Controller", group = "Axis 4"));
    parameter Real ks4 = 0.5 "Gain of speed controller" annotation(Dialog(tab = "Controller", group = "Axis 4"));
    parameter SI.Time Ts4 = 0.05 "Time constant of integrator of speed controller" annotation(Dialog(tab = "Controller", group = "Axis 4"));
    parameter Real kp5 = 5 "Gain of position controller" annotation(Dialog(tab = "Controller", group = "Axis 5"));
    parameter Real ks5 = 0.5 "Gain of speed controller" annotation(Dialog(tab = "Controller", group = "Axis 5"));
    parameter SI.Time Ts5 = 0.05 "Time constant of integrator of speed controller" annotation(Dialog(tab = "Controller", group = "Axis 5"));
    parameter Real kp6 = 5 "Gain of position controller" annotation(Dialog(tab = "Controller", group = "Axis 6"));
    parameter Real ks6 = 0.5 "Gain of speed controller" annotation(Dialog(tab = "Controller", group = "Axis 6"));
    parameter SI.Time Ts6 = 0.05 "Time constant of integrator of speed controller" annotation(Dialog(tab = "Controller", group = "Axis 6"));
    IndustrialRobot.Components.MechanicalStructure mechanics(mLoad = mLoad, rLoad = rLoad, g = g, toB.animation = false, toA.animation = false) annotation(Placement(transformation(extent = {{35, -35}, {95, 25}}, rotation = 0, origin = {-15, 5}), visible = true));
    Modelica.Mechanics.MultiBody.Examples.Systems.RobotR3.Utilities.PathPlanning6 pathPlanning(naxis = 6, angleBegDeg = {startAngle1, startAngle2, startAngle3, startAngle4, startAngle5, startAngle6}, angleEndDeg = {endAngle1, endAngle2, endAngle3, endAngle4, endAngle5, endAngle6}, speedMax = refSpeedMax, accMax = refAccMax, startTime = refStartTime, swingTime = refSwingTime) annotation(Placement(transformation(extent = {{-25, -75}, {-5, -55}}, rotation = -180, origin = {-55, 15}), visible = true));
    Modelica.Mechanics.MultiBody.Examples.Systems.RobotR3.Utilities.AxisType1 axis1(w = 4590, ratio = -105, c = 43, cd = 0.005, Rv0 = 0.4, Rv1 = 0.13 / 160, kp = kp1, ks = ks1, Ts = Ts1) annotation(Placement(transformation(extent = {{-25, -75}, {-5, -55}}, rotation = 0, origin = {-15, 15}), visible = true));
    Modelica.Mechanics.MultiBody.Examples.Systems.RobotR3.Utilities.AxisType1 axis2(w = 5500, ratio = 210, c = 8, cd = 0.01, Rv1 = 0.1 / 130, Rv0 = 0.5, kp = kp2, ks = ks2, Ts = Ts2) annotation(Placement(transformation(extent = {{-25, -55}, {-5, -35}}, rotation = 0, origin = {-15, 15}), visible = true));
    Modelica.Mechanics.MultiBody.Examples.Systems.RobotR3.Utilities.AxisType1 axis3(w = 5500, ratio = 60, c = 58, cd = 0.04, Rv0 = 0.7, Rv1 = 0.2 / 130, kp = kp3, ks = ks3, Ts = Ts3) annotation(Placement(transformation(extent = {{-25, -35}, {-5, -15}}, rotation = 0, origin = {-15, 15}), visible = true));
    Modelica.Mechanics.MultiBody.Examples.Systems.RobotR3.Utilities.AxisType2 axis4(k = 0.2365, w = 6250, D = 0.55, J = 0.00016, ratio = -99, Rv0 = 21.8, Rv1 = 9.8, peak = 26.7 / 21.8, kp = kp4, ks = ks4, Ts = Ts4) annotation(Placement(transformation(extent = {{-25, -15}, {-5, 5}}, rotation = 0, origin = {-15, 15}), visible = true));
    Modelica.Mechanics.MultiBody.Examples.Systems.RobotR3.Utilities.AxisType2 axis5(k = 0.2608, w = 6250, D = 0.55, J = 0.00018, ratio = 79.2, Rv0 = 30.1, Rv1 = 0.03, peak = 39.6 / 30.1, kp = kp5, ks = ks5, Ts = Ts5) annotation(Placement(transformation(extent = {{-25, 5}, {-5, 25}}, rotation = 0, origin = {-15, 15}), visible = true));
    Modelica.Mechanics.MultiBody.Examples.Systems.RobotR3.Utilities.AxisType2 axis6(k = 0.0842, w = 7400, D = 0.27, J = 4.3e-05, ratio = -99, Rv0 = 10.9, Rv1 = 3.92, peak = 16.8 / 10.9, kp = kp6, ks = ks6, Ts = Ts6) annotation(Placement(transformation(extent = {{-25, 25}, {-5, 45}}, rotation = 0, origin = {-15, 15}), visible = true));
  protected
    Modelica.Mechanics.MultiBody.Examples.Systems.RobotR3.Utilities.ControlBus controlBus annotation(Placement(transformation(origin = {-80, 0}, extent = {{-20, -20}, {20, 20}}, rotation = 90), visible = true));
  equation
    connect(controlBus, pathPlanning.controlBus) annotation(Line(visible = true, origin = {-70, 53.333}, points = {{-10, -53.333}, {-10, 26.667}, {20, 26.667}}, color = {255, 204, 51}, thickness = 0.5));
    connect(controlBus.axisControlBus1, axis1.axisControlBus) annotation(Line(visible = true, points = {{-13.333, 33.333}, {-3.333, 33.333}, {-3.333, -16.667}, {26.667, -16.667}}, color = {255, 204, 51}, thickness = 0.5, origin = {-66.667, -33.333}));
    connect(axis1.flange, mechanics.axis1) annotation(Line(visible = true, points = {{-24.625, -13.75}, {5.375, -13.75}, {5.375, 13.75}, {13.875, 13.75}}, origin = {4.625, -36.25}, color = {64, 64, 64}));
    connect(axis2.flange, mechanics.axis2) annotation(Line(visible = true, points = {{-19.625, -8.25}, {0.375, -8.25}, {0.375, 8.25}, {18.875, 8.25}}, origin = {-0.375, -21.75}, color = {64, 64, 64}));
    connect(axis3.flange, mechanics.axis3) annotation(Line(visible = true, points = {{-14.625, -2.75}, {-4.625, -2.75}, {-4.625, 2.75}, {23.875, 2.75}}, origin = {-5.375, -7.25}, color = {64, 64, 64}));
    connect(axis4.flange, mechanics.axis4) annotation(Line(visible = true, points = {{-19.625, 2.75}, {-9.625, 2.75}, {-9.625, -2.75}, {18.875, -2.75}}, origin = {-0.375, 7.25}, color = {64, 64, 64}));
    connect(axis5.flange, mechanics.axis5) annotation(Line(visible = true, points = {{-19.625, 8.25}, {0.375, 8.25}, {0.375, -8.25}, {18.875, -8.25}}, origin = {-0.375, 21.75}, color = {64, 64, 64}));
    connect(axis6.flange, mechanics.axis6) annotation(Line(visible = true, points = {{-24.625, 13.75}, {5.375, 13.75}, {5.375, -13.75}, {13.875, -13.75}}, origin = {4.625, 36.25}, color = {64, 64, 64}));
    connect(controlBus.axisControlBus2, axis2.axisControlBus) annotation(Line(visible = true, points = {{-20, 15}, {0, 15}, {0, -15}, {20, -15}}, color = {255, 204, 51}, thickness = 0.5, origin = {-60, -15}));
    connect(controlBus.axisControlBus3, axis3.axisControlBus) annotation(Line(visible = true, points = {{-25, 5}, {5, 5}, {5, -5}, {15, -5}}, color = {255, 204, 51}, thickness = 0.5, origin = {-55, -5}));
    connect(controlBus.axisControlBus4, axis4.axisControlBus) annotation(Line(visible = true, points = {{-25, -5}, {5, -5}, {5, 5}, {15, 5}}, color = {255, 204, 51}, thickness = 0.5, origin = {-55, 5}));
    connect(controlBus.axisControlBus5, axis5.axisControlBus) annotation(Line(visible = true, points = {{-20, -15}, {0, -15}, {0, 15}, {20, 15}}, color = {255, 204, 51}, thickness = 0.5, origin = {-60, 15}));
    connect(controlBus.axisControlBus6, axis6.axisControlBus) annotation(Line(visible = true, points = {{-16.667, -33.333}, {-6.667, -33.333}, {-6.667, 16.667}, {23.333, 16.667}}, color = {255, 204, 51}, thickness = 0.5, origin = {-63.333, 33.333}));
    annotation(__Wolfram_useDSS = true, Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -70}, {100, 100}}, grid = {10, 10}, initialScale = 0.1)), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}, grid = {0.5, 0.5}, initialScale = 0.1), graphics = {Rectangle(visible = true, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid, extent = {{-99.5, -100}, {100, 100}}), Bitmap(visible = true, fileName = "modelica://IndustrialRobot/DocumentationFiles/robot_kr15.bmp", imageSource = "", extent = {{-75.5, -96.75}, {87, 98.25}}), Text(visible = true, textColor = {0, 0, 255}, extent = {{-111.5, 100}, {108.5, 130}}, textString = "%name", fontName = "Arial"), Text(visible = true, extent = {{-104.5, -128}, {115, -104}}, textString = "mLoad=%mLoad", fontName = "Arial")}), Documentation(info = "<html>
<p>This is a six degrees of freedom robot with path planning, controllers, motors, brakes, gears, and mechanics.</p>

<h4>Simulation</h4>

<p>To simulate the robot and view a 3D animation of it, follow the steps below:</p>


<p>Click the Simulate button: <img src=\"modelica://IndustrialRobot/DocumentationFiles/play.svg\" style=\"width:20px\" alt=\"simulate\" /></p>
<p>Click the Animate button: <img src=\"modelica://IndustrialRobot/DocumentationFiles/animate.svg\" style=\"width:20px\" alt=\"animate\" /></p>


<p style=\"text-align: center;\"><img src=\"modelica://IndustrialRobot/DocumentationFiles/animation.png\"  alt=\"\" /></p>

<h4>Instant Plotting</h4>

<p>Compare the planned trajectory for one of the robot's axes with the simulated controlled path by plotting their respective variables.</p>



<p>A preffered Model plot will automatically be displayed, showing reference speed values for the different axes, as well as the actual speed values. There are multiple Model plots showing only individual axes. Opening the Model plot Axis1 in Axes velocity vs reference group, you will see the plot below.</p>

<p style=\"text-align: center;\"><img src=\"modelica://IndustrialRobot/DocumentationFiles/plot.png\" style=\"width:600px\" alt=\"\" /></p>


<h4>Real-Time Simulation</h4>

<p>It is possible to synchronize the simulation with real time. To do this, click the Settings tab in Simulation Center and choose the Synchronize with real-time option.</p>

<p style=\"text-align: center;\"><img src=\"modelica://IndustrialRobot/DocumentationFiles/realTime.png\" style=\"width:564px\" alt=\"\" /></p>

</div></html>", figures = {Figure(title = "Axis angles", identifier = "ba664", preferred = true, plots = {Plot(curves = {Curve(y = pathPlanning.controlBus.axisControlBus1.angle, legend = "Axis 1"), Curve(y = pathPlanning.controlBus.axisControlBus2.angle, legend = "Axis 2"), Curve(y = pathPlanning.controlBus.axisControlBus3.angle, legend = "Axis 3"), Curve(y = pathPlanning.controlBus.axisControlBus4.angle, legend = "Axis 4"), Curve(y = pathPlanning.controlBus.axisControlBus5.angle, legend = "Axis 5"), Curve(y = pathPlanning.controlBus.axisControlBus6.angle, legend = "Axis 6")})}), Figure(title = "Angular velocities", identifier = "588d4", preferred = true, plots = {Plot(curves = {Curve(y = pathPlanning.controlBus.axisControlBus1.speed, legend = "Axis 1"), Curve(y = pathPlanning.controlBus.axisControlBus2.speed, legend = "Axis 2"), Curve(y = pathPlanning.controlBus.axisControlBus3.speed, legend = "Axis 3"), Curve(y = pathPlanning.controlBus.axisControlBus4.speed, legend = "Axis 4"), Curve(y = pathPlanning.controlBus.axisControlBus5.speed, legend = "Axis 5"), Curve(y = pathPlanning.controlBus.axisControlBus6.speed, legend = "Axis 6")})}), Figure(title = "Angular accelerations", identifier = "f4443", plots = {Plot(curves = {Curve(y = pathPlanning.controlBus.axisControlBus1.acceleration, legend = "Axis 1"), Curve(y = pathPlanning.controlBus.axisControlBus2.acceleration, legend = "Axis 2"), Curve(y = pathPlanning.controlBus.axisControlBus3.acceleration, legend = "Axis 3"), Curve(y = pathPlanning.controlBus.axisControlBus4.acceleration, legend = "Axis 4"), Curve(y = pathPlanning.controlBus.axisControlBus5.acceleration, legend = "Axis 5"), Curve(y = pathPlanning.controlBus.axisControlBus6.acceleration, legend = "Axis 6")})}), Figure(title = "Axis1", identifier = "fc9b3", group = "Axes velocity vs reference", plots = {Plot(curves = {Curve(y = pathPlanning.controlBus.axisControlBus1.speed, legend = "Angular Velocity"), Curve(y = pathPlanning.controlBus.axisControlBus1.speed_ref, legend = "Reference")})}), Figure(title = "Axis2", identifier = "535f1", group = "Axes velocity vs reference", plots = {Plot(curves = {Curve(y = pathPlanning.controlBus.axisControlBus2.speed, legend = "Angular Velocity"), Curve(y = pathPlanning.controlBus.axisControlBus2.speed_ref, legend = "Reference")})}), Figure(title = "Axis3", identifier = "6fde9", group = "Axes velocity vs reference", plots = {Plot(curves = {Curve(y = pathPlanning.controlBus.axisControlBus3.speed, legend = "Angular Velocity"), Curve(y = pathPlanning.controlBus.axisControlBus3.speed_ref, legend = "Reference")})}), Figure(title = "Axis4", identifier = "71d37", group = "Axes velocity vs reference", plots = {Plot(curves = {Curve(y = pathPlanning.controlBus.axisControlBus4.speed, legend = "Angular Velocity"), Curve(y = pathPlanning.controlBus.axisControlBus4.speed_ref, legend = "Reference")})}), Figure(title = "Axis5", identifier = "1384f", group = "Axes velocity vs reference", plots = {Plot(curves = {Curve(y = pathPlanning.controlBus.axisControlBus5.speed, legend = "Angular Velocity"), Curve(y = pathPlanning.controlBus.axisControlBus5.speed_ref, legend = "Reference")})}), Figure(title = "Axis6", identifier = "1d4d8", group = "Axes velocity vs reference", plots = {Plot(curves = {Curve(y = pathPlanning.controlBus.axisControlBus6.speed, legend = "Angular Velocity"), Curve(y = pathPlanning.controlBus.axisControlBus6.speed_ref, legend = "Reference")})}), Figure(title = "Axis1", identifier = "fc9b37", group = "Axes angle vs reference", plots = {Plot(curves = {Curve(y = pathPlanning.controlBus.axisControlBus1.angle, legend = "Angle"), Curve(y = pathPlanning.controlBus.axisControlBus1.angle_ref, legend = "Reference")})}), Figure(title = "Axis2", identifier = "535f15", group = "Axes angle vs reference", plots = {Plot(curves = {Curve(y = pathPlanning.controlBus.axisControlBus2.angle, legend = "Angle"), Curve(y = pathPlanning.controlBus.axisControlBus2.angle_ref, legend = "Reference")})}), Figure(title = "Axis3", identifier = "6fde93", group = "Axes angle vs reference", plots = {Plot(curves = {Curve(y = pathPlanning.controlBus.axisControlBus3.angle, legend = "Angle"), Curve(y = pathPlanning.controlBus.axisControlBus3.angle_ref, legend = "Reference")})}), Figure(title = "Axis4", identifier = "71d377", group = "Axes angle vs reference", plots = {Plot(curves = {Curve(y = pathPlanning.controlBus.axisControlBus4.angle, legend = "Angle"), Curve(y = pathPlanning.controlBus.axisControlBus4.angle_ref, legend = "Reference")})}), Figure(title = "Axis5", identifier = "1384f6", group = "Axes angle vs reference", plots = {Plot(curves = {Curve(y = pathPlanning.controlBus.axisControlBus5.angle, legend = "Angle"), Curve(y = pathPlanning.controlBus.axisControlBus5.angle_ref, legend = "Reference")})}), Figure(title = "Axis6", identifier = "1d4d82", group = "Axes angle vs reference", plots = {Plot(curves = {Curve(y = pathPlanning.controlBus.axisControlBus6.angle, legend = "Angle"), Curve(y = pathPlanning.controlBus.axisControlBus6.angle_ref, legend = "Reference")})}), Figure(title = "Motor currents", identifier = "a7d58", plots = {Plot(curves = {Curve(y = pathPlanning.controlBus.axisControlBus1.current, legend = "Axis 1"), Curve(y = pathPlanning.controlBus.axisControlBus2.current, legend = "Axis 2"), Curve(y = pathPlanning.controlBus.axisControlBus3.current, legend = "Axis 3"), Curve(y = pathPlanning.controlBus.axisControlBus4.current, legend = "Axis 4"), Curve(y = pathPlanning.controlBus.axisControlBus5.current, legend = "Axis 5"), Curve(y = pathPlanning.controlBus.axisControlBus6.current, legend = "Axis 6")})}), Figure(title = "XY position", identifier = "e90d2", plots = {Plot(curves = {Curve(x = mechanics.load.frame_b.r_0[1], y = mechanics.load.frame_b.r_0[2], legend = "Load X vs Y Position")})}), Figure(title = "XZ position", identifier = "ee230", plots = {Plot(curves = {Curve(x = mechanics.load.frame_b.r_0[1], y = mechanics.load.frame_b.r_0[3], legend = "Load X vs Z Position")})}), Figure(title = "ZY position", identifier = "a51e1", plots = {Plot(curves = {Curve(x = mechanics.load.frame_b.r_0[3], y = mechanics.load.frame_b.r_0[2], legend = "Load Z vs Y Position")})})}), experiment(StartTime = 0.0, StopTime = 2, NumberOfIntervals = 500, __Wolfram_Algorithm = "cvodes", Tolerance = 1e-05, __Wolfram_SynchronizeWithRealTime = false), __Wolfram(ControlPanels(Panel(identifier = "clever-tao", title = "Model parameters", elements = {Slider(variable = mLoad, label = "Mass of load", min = 5, max = 30)})), Animation(cameras = {Camera(name = "Default", center = {-0.03231929652859759, 0.3241546352458354, -0.1158191172599261}, distance = 6.155761911316205, rotation = {-0.1161165091085138, 0.4819411266260691, 0.06462356470302268, 0.8660678389502301})}, windows = {Window(name = "Default", groundGrid = false, camera = "Default")})), preferredView = "info");
  end FullRobot;
  annotation(__Wolfram(openClass = IndustrialRobot.FullRobot), uses(Modelica(version = "4.0.0")), Icon(coordinateSystem(extent = {{-100, -100}, {100, 100}}, preserveAspectRatio = true, initialScale = 0.1, grid = {10, 10}), graphics = {Rectangle(visible = true, lineColor = {111, 0, 0}, fillColor = {255, 0, 0}, fillPattern = FillPattern.HorizontalCylinder, lineThickness = 4, extent = {{-90, -90}, {90, 90}}, radius = 25), Text(visible = true, origin = {12, 20.833}, textColor = {255, 255, 255}, extent = {{-102, -70.833}, {78, 29.167}}, textString = "Ex", fontName = "Arial")}), Documentation(info = "<html></html>"));
end IndustrialRobot;
